<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsTrackerSplit.php 1239 2005-01-17 17:39:11Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal/StatisticsTracker.php';
require_once maintenancePath . '/libraries/pear/Date.php';
 
/**
 * The data access layer code for summarising raw data into statistics, for 
 * the Tracker module, when split tables are in use
 */
class Dal_Statistics_Tracker_Split extends Dal_Statistics_Tracker
{

    /**
     * The constructor method.
     *
     * @uses Dal_Statistics_Tracker::Dal_Statistics_Tracker()
     */
    function Dal_Statistics_Tracker_Split()
    {
        parent::Dal_Statistics_Tracker();
    }
    
    /**
     * The singleton method to create or return a single instance of the class.
     *
     * @static
     * @return Dal_Statistics_Tracker_Split The returned Dal_Statistics_Tracker_Split object.
     */
    function &singleton() {
        static $instance;
        if (!isset($instance)) {
            $instance = new Dal_Statistics_Tracker_Split();
        }
        return $instance;
    }
    
    /**
     * A method to find the last time that maintenance statistics was run.
     *
     * @param integer $type The update type that occurred - that is, 0 if
     *                      the update was done on the basis of the 
     *                      operation interval, or 1 if the update was
     *                      done on the basis of the hour.
     * @param Date $now An optional Date, used to specify the "current time", and
     *                  to limit the method to only look for past maintenance
     *                  statistics runs before this date. Normally only used
     *                  to assist with re-generation of statistics in the event
     *                  of faulty raw tables.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function getMaintenanceStatisticsLastRunInfo($type, $now = null)
    {
        $table = $this->conf['table']['data_raw_tracker_impression'] . '_' . date('Ymd');
        return $this->_getMaintenanceStatisticsLastRunInfo($type, $table, $now);
    }
    
    /**
     * A method to delete old (ie. summarised) raw data.
     *
     * @param Date $summarisedTo The date/time up to which data have been summarised (i.e. if there
     *                           is no newer data than this date (minus any compact_stats_grace window)
     *                           then the raw table will be dropped, unless required by the tracking
     *                           module, where installed).
     * @return integer The number of raw tables dropped.
     */
    function deleteOldData($summarisedTo)
    {
        $now = new Date();  // Current day
        $deleteDate = $summarisedTo;
        if ($this->conf['maintenance']['compact_stats_grace'] > 0) {
            $deleteDate->subtractSeconds((int) $this->conf['maintenance']['compact_stats_grace']);
        }
        $tables = 0;
        // As split tables are in use, look over all possible data_raw_tracker_impression tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_tracker_impression']}_%'";
        $this->execute($query, $result);
        if ($this->numberOfRows($result) > 0) {
            while ($row = $this->fetchResultAsArray($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    // Is this today's table?
                    if (($date->getYear() == $now->getYear()) &&
                        ($date->getMonth() == $now->getMonth()) &&
                        ($date->getDay() == $now->getDay())) {
                        // Don't drop today's table
                        continue;
                    }
                    $table = $this->conf['table']['data_raw_tracker_impression'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'
                        LIMIT 1";
                    debug("Selecting non-summarised (later than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                          "') tracker impressions from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && ($this->numberOfRows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No non-summarised tracker impressions in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        // As split tables are in use, look over all possible data_raw_tracker_variable_value tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_tracker_variable_value']}_%'";
        $this->execute($query, $result);
        if ($this->numberOfRows($result) > 0) {
            while ($row = $this->fetchResultAsArray($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    // Is this today's table?
                    if (($date->getYear() == $now->getYear()) &&
                        ($date->getMonth() == $now->getMonth()) &&
                        ($date->getDay() == $now->getDay())) {
                        // Don't drop today's table
                        continue;
                    }
                    $table = $this->conf['table']['data_raw_tracker_variable_value'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'
                        LIMIT 1";
                    debug("Selecting non-summarised (later than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                          "') tracker variable values from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && ($this->numberOfRows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No non-summarised tracker variable values in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        // As split tables are in use, look over all possible data_raw_tracker_click tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_tracker_click']}_%'";
        $this->execute($query, $result);
        if ($this->numberOfRows($result) > 0) {
            while ($row = $this->fetchResultAsArray($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    // Is this today's table?
                    if (($date->getYear() == $now->getYear()) &&
                        ($date->getMonth() == $now->getMonth()) &&
                        ($date->getDay() == $now->getDay())) {
                        // Don't drop today's table
                        continue;
                    }
                    $table = $this->conf['table']['data_raw_tracker_click'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'
                        LIMIT 1";
                    debug("Selecting non-summarised (later than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
                          "') tracker clicks from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && ($this->numberOfRows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No non-summarised tracker clicks in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        return $tables;
    }
    
}

?>
