<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: all_tests.php 1239 2005-01-17 17:39:11Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 */

/**
 * A script for running the Maintenance Data Access Layer tests. Designed to
 * be included in a main, project-wide all_tests.php file, or to be run by
 * itself.
 */

// Configure the installation paths
if (!defined('MAX_PATH')) {
    define('MAX_PATH', preg_replace('#/maintenance/libraries/max/Dal/tests#', '', dirname(__FILE__)));
}
if (!defined('maintenancePath')) {
    define('maintenancePath', MAX_PATH . '/maintenance');
}

// Include the simpletest classes, if required
require_once MAX_PATH . '/libraries/pear/simpletest/unit_tester.php';
require_once MAX_PATH . '/libraries/pear/simpletest/reporter.php';

// Load the maintenance config file, if required
if (is_null($conf)) {
    $conf = parse_ini_file(maintenancePath . '/default.conf.ini', true);
    $GLOBALS['_MAX']['CONF'] =& $conf;
}

// Set the PEAR Date class location
ini_set('include_path', maintenancePath . '/libraries/pear');

// Define the week to start on Sunday (0) so that the Date and Date_Calc
// classes agree on what day is the start of the week, if required
if (!defined(DATE_CALC_BEGIN_WEEKDAY)) {
    define('DATE_CALC_BEGIN_WEEKDAY', 0);
}

global $testStart, $testEnd;
if (!isset($testStart)) {
    // Output the HTML header
    include MAX_PATH . '/tests/header.html';
    // Record the test start time
    $testStart = time();
    $runTime = true;
}

// Run the tests on the Maintenance Data Access Layer that do NOT need a database
$test = &new GroupTest('Maintenance Data Access Layer: Base Tests');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfDal.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatistics.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatisticsCommon.php');
$test->run(new HtmlReporter());

// Configure a connection to the test database
require_once maintenancePath . '/libraries/max/Dal.php';
$dal = Dal::singleton();
$query = "DROP DATABASE IF EXISTS max_simpletest";
$dal->execute($query, $result);
$query = "CREATE DATABASE max_simpletest";
$dal->execute($query, $result);
// Select the database, so that it is used for all tests from this point on
if (!mysql_select_db('max_simpletest', $dal->dbConnection)) {
    $this->fail();
}

// Run the tests on the Maintenance Data Access Layer that DO need a database
$test = &new GroupTest('Maintenance Data Access Layer: Data Tests');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfTableCore.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatisticsAdServer.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatisticsAdServerSplit.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatisticsTracker.php');
$test->addTestFile(maintenancePath . '/libraries/max/Dal/tests/TestOfStatisticsTrackerSplit.php');
$test->run(new HtmlReporter());

// Drop the test database
$query = "DROP DATABASE max_simpletest";
$dal->execute($query, $result);

if ($runTime) {
    // Calculate and display the test run time
    $testEnd = time();
    $time = $testEnd - $testStart;
    echo '<p>Tests took ' . $time . ' seconds</p>';
    // Output the HTML footer
    include MAX_PATH . '/tests/footer.html';
}

?>
