<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsAdServer.php 1239 2005-01-17 17:39:11Z andrew $
*/

/**
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal/Statistics.php';
require_once maintenancePath . '/libraries/max/Maintenance/StatisticsCommon.php';

/**
 * A class for summarising raw data into the various statistics that Max needs
 * for the AdServer module.
 */
class Maintenance_Statistics_AdServer extends Maintenance_Statistics_Common 
{

    /**
     * The constructor method.
     *
     * @param Date $now Optional Date representing the current date/time. Normally
     *                  just used for testing the class constructor, or for 
     *                  faking the current time in order to re-generate past statistics,
     *                  in the case of faulty raw data tables.
     * @param Date $lastUpdated Optional Date representing the date/time the
     *                          statistics were last updated. Only used when
     *                          regenerating statistics.
     */
    function Maintenance_Statistics_AdServer($now = null, $lastUpdated = null)
    {
        $this->module = 'AdServer';
        parent::Maintenance_Statistics_Common($now, $lastUpdated);
    }
    
    /**
     * A method to summarise raw data into the intermediate statistics tables.
     */
    function summariseIntermediate()
    {
        if (($this->updateHourly) && (!$this->updateUsingOI)) {
            // Summarise the intermediate stats by the hour
            $start = $this->lastDateHourly;
            $start->addSeconds(1);
            while (Date::compare($start, $this->upToDateHourly) < 0) {
                // Calculate the end of the hour
                $end = $start;
                $end->addSeconds(3600 - 1); // Plus one hour
                // Summarise the requests
                $this->_summariseIntermediateRequests($start, $end);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($start, $end);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($start, $end);
                if ($this->conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($start, $end);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($start, $end);
                // Go to the next hour
                $start->addSeconds(3600);
            }
        } else if (($this->updateOI) && ($this->updateUsingOI)) {
            // Summarise the intermediate stats by the operation interval
            $start = $this->lastDateOI;
            $start->addSeconds(1);
            while (Date::compare($start, $this->upToDateOI) < 0) {
                // Calcuate the end of the operation interval
                $end = $start;
                $end->addSeconds(($this->conf['maintenance']['operation_interval'] * 60) - 1); // Plus one OI
                // Summarise the requests
                $this->_summariseIntermediateRequests($start, $end);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($start, $end);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($start, $end);
                if ($this->conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($start, $end);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($start, $end);
                // Go to the next operation interval
                $start->addSeconds($this->conf['maintenance']['operation_interval'] * 60);
            }
        }
    }
    
    /** 
     * A private method to summarise the ad requests, ready for insertion into
     * intermediate statistics tables.
     *
     * @private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateRequests($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising requests for the intermediate tables.\n";
        debug('Summarising requests for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising requests between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' . 
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        debug('Summarising requests between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' . 
              $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseRequests($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows request rows in $runTime seconds.\n";
        debug("Summarised $rows request rows in $runTime seconds", PEAR_LOG_DEBUG);
    }
    
    /** 
     * A private method to summarise the ad impressions, ready for insertion into
     * intermediate statistics tables.
     *
     * @private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateImpressions($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising impressions for the intermediate tables.\n";
        debug('Summarising impressions for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising impressions between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        debug('Summarising impressions between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
              $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseImpressions($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows impression rows in $runTime seconds.\n";
        debug("Summarised $rows impression rows in $runTime seconds", PEAR_LOG_DEBUG);
    }
    
    /** 
     * A private method to summarise the ad clicks, ready for insertion into
     * intermediate statistics tables.
     *
     * @private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateClicks($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising clicks for the intermediate tables.\n";
        debug('Summarising clicks for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising clicks between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        debug('Summarising clicks between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
              $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseClicks($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows click rows in $runTime seconds.\n";
        debug("Summarised $rows click rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /** 
     * A private method to summarise the ad connections, ready for insertion into
     * intermediate statistics tables.
     *
     * @private
     * @param Date $start The start date of the data to summarise.
     * @param Date $end The end date of the data to summarise.
     */
    function _summariseIntermediateConnections($start, $end)
    {
        $startTime = time();
        $this->report .= "Summarising connections for the intermediate tables.\n";
        debug('Summarising connections for the intermediate tables', PEAR_LOG_DEBUG);
        $this->report .= 'Summarising connections between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                         $end->format('%Y-%m-%d %H:%M:%S') . ".\n";
        debug('Summarising connections between ' . $start->format('%Y-%m-%d %H:%M:%S') . ' and ' . 
              $end->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $this->dal->summariseConnections($start, $end);
        $runTime = time() - $startTime;
        $this->report .= "Summarised $rows connection rows in $runTime seconds.\n";
        debug("Summarised $rows connection rows in $runTime seconds", PEAR_LOG_DEBUG);
    }
    
    /** 
     * A private method to save the summarised request/impressions/clicks/connections
     * into the intermediate tables.
     *
     * @private
     * @param Date $start The start date of the data to save.
     * @param Date $end The end date of the data to save.
     */
    function _saveIntermediateSummaries($start, $end)
    {
        if ($this->conf['modules']['Tracker']) {
            $this->report .= "Saving request, impression, click and connection data into the intermediate tables.\n";
            debug('Saving request, impression, click and connection data into the intermediate tables', PEAR_LOG_DEBUG);
        } else {
            $this->report .= "Saving request, impression, click data into the intermediate tables.\n";
            debug('Saving request, impression, click data into the intermediate tables', PEAR_LOG_DEBUG);
        }
        $this->dal->saveIntermediate($start, $end);
    }
    
    /**
     * A method to summarise intermediate statistics into summary (final) statistics
     * tables, where required.
     */
    function summariseFinal()
    {
        if ($this->updateOI) {
            // Update the zone impression history table
            $this->report .= 'Updating the zone impression history table for data after ' . 
                             $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".\n";
            debug('Updating the zone impression history tables for data after ' . 
                  $this->lastDateOI->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
            $start = $this->lastDateOI;
            $start->addSeconds(1);
            $this->dal->saveHistory($start, $this->upToDateOI);
        }
        if ($this->updateHourly) {
            // Update the hourly summary table
            $this->report .= 'Updating the hourly summary table for data after ' . 
                             $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".\n";
            debug('Updating the hourly summary table for data after ' . 
                  $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
            $start = $this->lastDateHourly;
            $start->addSeconds(1);
            $this->dal->saveSummary($start, $this->upToDateHourly);
        }
    }

    /**
     * A method to decrement the remaining impressions, clicks and conversions
     * of campaigns based on the saved, summarised data.
     */
    function decrementCampaigns()
    {
        if ($this->updateOI) {
            $this->report .= "Decrementing the campaign impressions, clicks and conversions.\n";
            $message .= 'Decrementing the campaign impressions, clicks and conversions.';
            debug($message);
            $from = $this->lastDateOI;
            $from->addSeconds(1);
            $this->dal->decrementCampaigns($this->report, $from, $this->upToDateOI);
        }
    }
    
}

?>
