#!/usr/bin/php -q
<?php

error_reporting(E_ALL ^ E_NOTICE ^ E_WARNING);

// Uncomment the following to enable PHP warnings when
// developing, as some of the phpAdsNew code that hasn't
// been rewritten yet and still generates warnings...
// error_reporting(E_ALL ^ E_NOTICE);

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: maintenance.php 1239 2005-01-17 17:39:11Z andrew $
*/

// Record the maintenance start time
$maintenanceStartTime = time();

// Deal with older versions of PHP
if (!function_exists('version_compare') || version_compare(phpversion(), "4.3.0", 'lt')) {
    include_once 'libraries/bc.php';
}

// Deal with PHP5
if (function_exists('version_compare') && version_compare(phpversion(), "5.0.0", 'ge')) {
    include_once 'libraries/5.php';
}

// +---------------------------------------+
// | Configure the maintenance environment |
// +---------------------------------------+

// Configure the installation path
define('maintenancePath', dirname(__FILE__));

// Load the maintenance config file
$conf = parse_ini_file(maintenancePath . '/default.conf.ini', true);
$GLOBALS['_MAX']['CONF'] =& $conf;

// Set the PEAR Date class location
ini_set('include_path', maintenancePath . '/libraries/pear');

// Define the week to start on Sunday (0) so that the Date and Date_Calc
// classes agree on what day is the start of the week
define('DATE_CALC_BEGIN_WEEKDAY', 0);

// Set time limit to 5 minutes and ignore user abort
if (!ini_get('safe_mode')) {
    @set_time_limit(300);
    @ignore_user_abort(true);
}

// Disable magic_quotes_runtime
set_magic_quotes_runtime(0);

// +---------------------------------------+
// | Load common functions and strings     |
// +---------------------------------------+

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/translationStrings.php';

debug('======================================================================', PEAR_LOG_INFO);
debug('Maintenance Started', PEAR_LOG_INFO);

// +---------------------------------------+
// | Check the operation interval vaule    |
// | Must be a factor or multiple of 60,   |
// | and a factor of (24 * 60)             |
// +---------------------------------------+

$oi = $conf['maintenance']['operation_interval'];
if ($oi < 1) {    
    debug("Operation interval is $oi which is less than one", PEAR_LOG_ERROR);
    $message  = 'Warning! Maintenance was unable to run - the operation interval' . "\n";
    $message .= 'found was ' . $this->options['operation_interval'] . ' which is less than one.' . "\n\n";
    $message .= 'The maintenance scripts need to be run at least once a week - every' . "\n" .
    $message .= '60 minutes is the normal setting.';
    phpAds_sendMail($conf['email']['admin'], '', 'Operation interval critical error!', $message);
    debug('Aborting script execution', PEAR_LOG_CRIT);
    exit();
} else if ($oi < 60) {
    if ((60 % $oi) != 0) {
        debug("Operation interval is $oi which is not a factor of 60", PEAR_LOG_ERROR);
        $message  = 'Warning! Maintenance was unable to run - the operation interval' . "\n";
        $message .= 'found was ' . $this->options['operation_interval'] . ' which is not a factor of 60.' . "\n\n";
        phpAds_sendMail($conf['email']['admin'], '', 'Operation interval critical error!', $message);
        debug('Aborting script execution', PEAR_LOG_CRIT);
        exit();
    }
} else if ($oi > 60) {
    if (($oi % 60) != 0) {
        debug("Operation interval is $oi which is not a multiple of 60", PEAR_LOG_ERROR);
        $message  = 'Warning! Maintenance was unable to run - the operation interval' . "\n";
        $message .= 'found was ' . $this->options['operation_interval'] . ' which is not a multiple of 60.' . "\n\n";
        phpAds_sendMail($conf['email']['admin'], '', 'Operation interval critical error!', $message);
        debug('Aborting script execution', PEAR_LOG_CRIT);
        exit();
    }
    if ($oi > 24 * 60) {
        debug("Operation interval is $oi which is longer than one week", PEAR_LOG_ERROR);
        $message  = 'Warning! Maintenance was unable to run - the operation interval' . "\n";
        $message .= 'found was ' . $this->options['operation_interval'] . ' which is longer than one week.' . "\n\n";
        $message .= 'The maintenance scripts need to be run at least once a week - every' . "\n" .
        $message .= '60 minutes is the normal setting.';
        phpAds_sendMail($conf['email']['admin'], '', 'Operation interval critical error!', $message);
        debug('Aborting script execution', PEAR_LOG_CRIT);
        exit();
    }    
}

// +---------------------------------------+
// | Check for lockfile                    |
// +---------------------------------------+

if ($conf['table']['split']) {
    debug('Checking for lockfile', PEAR_LOG_INFO);
    $attempt = 0;
    // Don't start if lockfile detected
    while (file_exists($conf['table']['lockfile'])) {
        // Pause for 30 secs
        sleep(30);
        debug('Lockfile exists, sleeping for 30 secs. Iteration ' . $attempt, PEAR_LOG_INFO);
        $attempt ++;
        if ($attempt > 2) {
            debug('More than 3 attempts, sending email to admin user', PEAR_LOG_ERROR);
            $message  = "Warning! Maintenance was unable to run - the lockfile was found\n";
            $message .= "in place while trying to start maintenance, even after 3 iterations.\n\n";
            $message .= "The lockfile used was: {$conf['table']['lockfile']}.\n\n";
            phpAds_sendMail($conf['email']['admin'], '', 'Lockfile altert!', $message);
            debug('Aborting script execution', PEAR_LOG_CRIT);
            exit();
        }
    }
    // Write lockfile so table splitting scripts will abort if they
    // attempt to start during a maintenance run
    debug('No lockfile exists, writing lockfile', PEAR_LOG_INFO);
    $fh = fopen($conf['table']['lockfile'], 'w');
}

// +---------------------------------------+
// | Connect to database for historical    |
// | maintenance code                      |
// +---------------------------------------+

debug('Creating phpAdsNew-style database connection', PEAR_LOG_DEBUG);
maintenance_dbConnect();
//phpAds_LoadDbConfig();

// +---------------------------------------+
// | Start maintenance                     |
// +---------------------------------------+

// Update the timestamp
$query = "
    UPDATE
        {$conf['table']['config']}
    SET
        maintenance_timestamp = UNIX_TIMESTAMP(NOW())";
$result = maintenance_dbQuery($query)
    or die('Query failed : ' . mysql_error());
debug('Updated the timestamp in the config table', PEAR_LOG_DEBUG);

// +---------------------------------------+
// | Maintenance Statistics                |
// +---------------------------------------+

debug('Running Maintenance Statistics', PEAR_LOG_INFO);
require_once maintenancePath . '/libraries/max/Maintenance/Statistics.php';
foreach ($conf['modules'] as $module => $installed) {
    if ($installed) {
        // Create an appropriate Maintenance_Statistics class
        Maintenance_Statistics::instance($module, $maintenanceStatistics);
        // Step 1: Summarise raw data into the intermediate tables
        $maintenanceStatistics->summariseIntermediate();
        // Step 2: Summarise intermediate data into summary tables
        $maintenanceStatistics->summariseFinal();
        // If the module is the AdServer module...
        if ($module == 'AdServer') {
            // Optional Step 2.5: Decrement the campaign limitations,
            // and activate/deactivate campaigns
            $maintenanceStatistics->decrementCampaigns();
        }
        // Step 3: Delete old raw data that has been summarised
        $maintenanceStatistics->deleteOldData();
        // Step 4: Update the log tables detailing the maintenance statistics run
        $maintenanceStatistics->logCompletion();
    }
}

// +---------------------------------------+
// | If it's midnight, do additional tasks |
// +---------------------------------------+

if (date('H') == 0) {
    debug('Running Midnight Maintenance Tasks', PEAR_LOG_INFO);
    include_once maintenancePath . '/maintenance-reports.php';
    include_once maintenancePath . '/maintenance-autotargeting.php';
    include_once maintenancePath . '/maintenance-optimization.php';
}

// +---------------------------------------+
// | Maintenance Priority                  |
// +---------------------------------------+

debug('Running Maintenance Priority', PEAR_LOG_INFO);
require_once maintenancePath . '/maintenance-priority.php';
phpAds_PriorityCalculate();

// +---------------------------------------+
// | Remove lockfile                       |
// +---------------------------------------+

if ($conf['table']['split']) {
    debug('Removing lockfile', PEAR_LOG_INFO);
    unlink($conf['table']['lockfile']);
}

// +---------------------------------------+
// | Log results to database               |
// +---------------------------------------+

// Record the maintenance end time
$maintenanceEndTime = time();

// Log the maintenance run
$elapsedTime = $maintenanceEndTime - $maintenanceStartTime;
$maintenanceStartTime = date("Y-m-d H:i:s", $maintenanceStartTime);
$maintenanceEndTime   = date("Y-m-d H:i:s", $maintenanceEndTime);
$query = "
    INSERT INTO {$conf['table']['log_maintenance']}
        (
            start_run,
            end_run,
            duration
        )
    VALUES
        (
            '$maintenanceStartTime',
            '$maintenanceEndTime',
            $elapsedTime
        )";
debug('Logging the maintenance run', PEAR_LOG_DEBUG);
$result = maintenance_dbQuery($query)
    or die('Query failed : ' . mysql_error());

debug('Maintenance Completed', PEAR_LOG_INFO);
    
?>
